#!/usr/bin/php
<?php

// torrent.php

{
	$columns_env = getenv('COLUMNS');
	define('COLUMNS', ($columns_env === false) ? 80 : intval($columns_env));
	
	define('ANSI', (PHP_OS !== "WINNT"));
}

function unhesc($sz) {
	return html_entity_decode(str_replace('&nbsp;', ' ', $sz));
}

function bytes($num) {
	if ($num < 1024) {
		return $num;
	} else if ($num < 1024*1024) {
		return number_format($num/1024, 0).'K';
	} else if ($num < 1024*1024*1024) {
		return number_format($num/(1024*1024), 0).'M';
	} else if ($num < 1024*1024*1024*1024) {
		return number_format($num/(1024*1024*1024), 1).'G';
	} else {
		return number_format($num/(1024*1024*1024*1024), 2).'T';
	}
}

function bytes_parse($str) {
	$size_num = floatval($str);
	
	if (strpos($str, 'KiB') !== false) {
		return $size_num * 1024;
	} else if (strpos($str, 'MiB') !== false) {
		return $size_num * 1024 * 1024;
	} else if (strpos($str, 'GiB') !== false) {
		return $size_num * 1024 * 1024 * 1024;
	} else {
		return $size_num; // bytes?
	}
}

function ansi($fg=true, $color=7, $bold=false) {
	
	$parts = [];
	if (! is_null($fg)) {
		$parts[] = (($fg?30:40) + intval($color));
	}
	if ($bold) {
		$parts[] = "1";
	}
	
	if (count($parts)) {
		return "\x1b[".implode(';', $parts)."m";
	} else {
		return "";
	}
}

function ansi_reset() {
	return "\x1b[39;49;0m";
}

//

class Torrent {
	public $title;
	public $url;
	public $seeders;
	public $leechers;
	
	public $category;
	
	public $date;
	public $size;
	public $username;
	
	public $flags;
	public $ansi;
	
	public function small($key) {
		$title_width = COLUMNS - 18;
	
		$ret = sprintf(
			"%2s %-${title_width}s %5s %3s %3s\n",
			$key,
			substr($this->title, 0, $title_width),
			bytes($this->size),
			$this->seeders,
			$this->leechers
		);
		
		if (ANSI && $this->ansi) {
			return ansi(true, $this->ansi, true).$ret.ansi_reset();
		
		} else {
			return $ret;
		}
	}
	
	public function big() {
		$ret = '';
		$ret .= "    Title: ".$this->title."\n";
		$ret .= " Category: ".$this->category."\n";
		$ret .= "     Size: ".bytes($this->size)."\n";
		$ret .= "    Added: ".$this->date." by ".$this->username."\n";
		$ret .= "    Flags: ".$this->flags."\n";
		$ret .= "      S/L: ".$this->seeders." / ".$this->leechers."\n";
		return $ret;
	}
};

//

interface ITorrentSource {
	
	/**
	 * @return \Torrent[]
	 */
	public function search($sz);
};

class NyaaSource implements ITorrentSource {
	public function search($sz) {
		$body = file_get_contents('http://www.nyaa.se/?page=search&term='.urlencode($sz).'&sort=2');
		
		$matches = [];
		preg_match_all('~<tr class="[^\"]*tlistrow[^\"]*">.*?</tr>~sm', $body, $matches);
		
		$ret = [];
		foreach($matches[0] as $match) {
			$ret[] = $this->parse($match);
		}
		return $ret;
	}
	
	protected function parse($html) {
		$t = new Torrent();		
		$matches = [];
		
		if (preg_match('~<tr class="([^"]+)">~', $html, $matches)) {
			if (strpos($matches[1], 'trusted') !== false) {
				$t->flags = 'Green (trusted user)';
				$t->ansi = 2;
			} elseif (strpos($matches[1], 'remake') !== false) {
				$t->flags = 'Red (remake / modified)';
				$t->ansi = 1;
			} elseif (strpos($matches[1], 'aplus') !== false) {
				$t->flags = 'Blue (Nyaa vouches for A+ quality)';
				$t->ansi = 4;
			}
		}
		
		if (preg_match('~cats=[\d_]+" title="([^"]+)"~', $html, $matches)) {
			$t->category = unhesc($matches[1]);
		}
		
		if (preg_match('~tid=\d+">([^<]+)<~', $html, $matches)) {
			$t->title = unhesc($matches[1]);
		}
		
		if (preg_match('~tlistsize">([^<]+)<~', $html, $matches)) {
			$t->size = bytes_parse(unhesc($matches[1]));
		}
		
		if (preg_match('~tlistsn">(\d+)<~', $html, $matches)) {
			$t->seeders = intval($matches[1]);
		}
		
		if (preg_match('~tlistln">(\d+)<~', $html, $matches)) {
			$t->leechers = intval($matches[1]);
		}
		
		if (preg_match('~<a href="([^"]+)" title="Download"~', $html, $matches)) {
			$t->url = unhesc($matches[1]);
		}
		
		// The following information is not retrieved
		$t->date = '.';
		$t->username = '.';
		
		return $t;
	}
};

class ThePirateBaySource implements ITorrentSource {
	public function search($sz) {
		$body = file_get_contents('https://thepiratebay.se/search/'.urlencode($sz).'/0/7/0');
		$matches = [];
		
		preg_match_all('~<td class="vertTh">.*?</tr>~ms', $body, $matches);
		
		$ret = [];
		foreach($matches[0] as $match) {
			$ret[] = $this->parse($match);
		}
		return $ret;
	}
	
	protected function parse($html) {
		$t = new Torrent();
		$matches = [];
		
		if (preg_match_all('~title="([^"]*)"~', $html, $matches)) {
			$t->title = unhesc(substr($matches[1][2], strlen('Details for ')));			
			if (in_array("Trusted", $matches[1])) {
				$t->flags = "Trusted Uploader";
				$t->ansi = 2;
			} elseif (in_array("VIP", $matches[1])) {
				$t->flags = "VIP Uploader";
				$t->ansi = 5;
			}
		}
		
		if (preg_match_all('~<a href="([^"]*)" title="Download this torrent using magnet">~', $html, $matches)) {
			$t->url = unhesc($matches[1][0]);
		}
		
		if (preg_match_all('~More from this category">([^<]+)<~', $html, $matches)) {
			$t->category = $matches[1][0].'/'.$matches[1][1];
		}
		
		if (preg_match_all('~<td align="right">(\d+)</td>~', $html, $matches)) {
			$t->seeders  = $matches[1][0];
			$t->leechers = $matches[1][1];
		}
		
		if (preg_match('~Uploaded ([^,]+), Size ([^,]+),~', $html, $matches)) {
			$t->date = unhesc($matches[1]);
			$t->size = bytes_parse(unhesc($matches[2]));
		}
		
		if (preg_match('~/user/([^/"]+)~', $html, $matches)) {
			$t->username = unhesc($matches[1]);
		}
		
		return $t;
	}
};

class TorrentSourceFactory {

	private function __construct() { } 

	/**
	 * @return \ITorrentSource
	 */
	public static function get($src) {
		switch(strtolower($src)) {
			case 'nyaa': return new NyaaSource();
			case 'tpb': return new ThePirateBaySource();
			default:
				die("Unknown torrent source \"${src}\".\n");
		}
	}
}

function usage() {
	return <<<EOD
Usage: ./torrent [options] "query"

Options: --nyaa   Search using nyaa
         --tpb    Search using The Pirate Bay [default]
EOD;
}

function main($args) {

	// Parse arguments

	if (! count($args)) {
		die(usage());
	}

	$query = array_pop($args);
	if (substr($query, 0, 2) == "--") {
		die(usage());
	}
	
	$source = 'tpb';
	foreach($args as $arg) switch (strtolower($arg)) {
	
		case '--nyaa': {
			$source = 'nyaa';
		} break;
		
		case '--tpb': {
			$source = 'tpb';
		} break;
	
		default: {
			echo "Unknown argument ".$arg."\n";
			die(usage());
		} break;
	
	}
	
	// Search
	
	$ts = TorrentSourceFactory::get($source);
	$results = $ts->search($query);
	
	// UI
	
	if (! count($results)) {
		die("No results for query \"${query}\".\n");
	}
	
	$STDIN = fopen('php://stdin', 'r');
	
	for(;;) {
		
		foreach($results as $id => $t) {
			echo $t->small(base_convert($id, 10, 36));
		}

		echo "select> ";
		$in = trim(fgets($STDIN));

		if (! strlen($in)) {
			echo "\nQuitting.\n";
			return;
		}
		
		$selected = base_convert($in, 36, 10);
		if (! array_key_exists($selected, $results)) {
			die("Invalid selection \"${in}\".\n");
		}

		echo "\n";
		echo $results[$selected]->big();

		echo "confirm> ";
		$in = trim(fgets($STDIN));

		if (strtolower(substr($in, 0, 1)) == 'y') {
			$cmd = sprintf('transmission-remote -a %s', escapeshellarg($t->url));
			shell_exec($cmd);
			return;
		}
		
		echo "\n";
		
	}

}

main(array_slice($_SERVER['argv'], 1));
